/****************************************************************************
**
** Copyright (C) 2000-2008 TROLLTECH ASA. All rights reserved.
**
** This file is part of the Opensource Edition of the Qtopia Toolkit.
**
** This software is licensed under the terms of the GNU General Public
** License (GPL) version 2.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
** 
** Adapted for Motorola MPx220 by SiliX
**
**
****************************************************************************/

#include "mpx220kbdhandler.h"

#ifdef QT_QWS_MPX220
#include <QScreen>
#include <QSocketNotifier>
//#include <QTimer>

#include "qscreen_qws.h"
#include "qwindowsystem_qws.h"
#include "qapplication.h"
#include "qnamespace.h"
//#include "qperformancelog.h"
#include <qtopialog.h>

#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <sys/vt.h>
#include <sys/kd.h>

#define VTACQSIG SIGUSR1
#define VTRELSIG SIGUSR2

static int vtQws = 0;

mpx220KbdHandler::mpx220KbdHandler()
{
    qWarning("Loaded mpx220 keypad plugin");

    setObjectName( "mpx220 Keypad Handler" );

    kbdFD = ::open("/dev/tty0", O_RDONLY | O_NDELAY, 0);
    if (kbdFD >= 0)
    {
        qWarning("Opened tty0 as keypad input");
        m_notify = new QSocketNotifier(kbdFD, QSocketNotifier::Read, this);
        connect(m_notify, SIGNAL(activated(int)), this, SLOT(readKbdData()));
    }
    else
    {
        qWarning("Cannot open tty0 for keypad (%s)", strerror(errno));
        return;
    }

    tcgetattr(kbdFD, &origTermData);
    struct termios termdata;
    tcgetattr(kbdFD, &termdata);

    ioctl(kbdFD, KDSKBMODE, K_RAW);

    termdata.c_iflag = (IGNPAR | IGNBRK) & (~PARMRK) & (~ISTRIP);
    termdata.c_oflag = 0;
    termdata.c_cflag = CREAD | CS8;
    termdata.c_lflag = 0;
    termdata.c_cc[VTIME]=0;
    termdata.c_cc[VMIN]=1;
    cfsetispeed(&termdata, 9600);
    cfsetospeed(&termdata, 9600);
    tcsetattr(kbdFD, TCSANOW, &termdata);

    connect(QApplication::instance(), SIGNAL(unixSignal(int)), this, SLOT(handleTtySwitch(int)));
    QApplication::instance()->watchUnixSignal(VTACQSIG, true);
    QApplication::instance()->watchUnixSignal(VTRELSIG, true);

    struct vt_mode vtMode;
    ioctl(kbdFD, VT_GETMODE, &vtMode);

    // let us control VT switching
    vtMode.mode = VT_PROCESS;
    vtMode.relsig = VTRELSIG;
    vtMode.acqsig = VTACQSIG;
    ioctl(kbdFD, VT_SETMODE, &vtMode);

    struct vt_stat vtStat;
    ioctl(kbdFD, VT_GETSTATE, &vtStat);
    vtQws = vtStat.v_active;

#if 0
    m_timer = new QTimer(this);
    m_timer->setSingleShot(true);
    connect(m_timer, SIGNAL(timeout()), this, SLOT(repeat()));
#endif
}

mpx220KbdHandler::~mpx220KbdHandler()
{
qWarning("destroy");
    if (kbdFD >= 0)
    {
        ioctl(kbdFD, KDSKBMODE, K_XLATE);
        tcsetattr(kbdFD, TCSANOW, &origTermData);
        ::close(kbdFD);
    }
}

void mpx220KbdHandler::handleTtySwitch(int sig)
{
    qWarning("TTySwitch");
	if (sig == VTACQSIG) {
        if (ioctl(kbdFD, VT_RELDISP, VT_ACKACQ) == 0) {
            qwsServer->enablePainting(true);
            qt_screen->restore();
            qwsServer->resumeMouse();
            qwsServer->refresh();
        }
    } else if (sig == VTRELSIG) {
        qwsServer->enablePainting(false);
        qt_screen->save();
        if(ioctl(kbdFD, VT_RELDISP, 1) == 0) {
            qwsServer->suspendMouse();
        } else {
            qwsServer->enablePainting(true);
        }
    }
}

void mpx220KbdHandler::readKbdData()
{
    unsigned char           buf[80];
    unsigned short          driverKeyCode;
    unsigned short          unicode;
    unsigned int            qtKeyCode;
    bool                    isPressed;
    Qt::KeyboardModifiers   modifiers = Qt::NoModifier;
//    qWarning("Read data");

    int n = ::read(kbdFD, buf, 1);
//    qWarning("Readed data %i",n);
    for (int loop = 0; loop < n; loop++)
    {
        driverKeyCode   = (unsigned short)buf[loop];
        qtKeyCode       = 0;
        unicode         = 0xffff;
        isPressed       = (driverKeyCode & 0x80) == 0;

//        qWarning("keypressed: code=%i(%s)" ,driverKeyCode ,isPressed ? "Down" : "Up") ;
        //QPerformanceLog("keypressed") << "code=" << driverKeyCode << " (" << (isPressed ? "Down" : "Up") << ")";

        switch (driverKeyCode & 0x7f)
        {
            // Keypad
            case 11: qtKeyCode = Qt::Key_0; unicode  = 0x30; break;
            case  2: qtKeyCode = Qt::Key_1; unicode  = 0x31; break;
            case  3: qtKeyCode = Qt::Key_2; unicode  = 0x32; break;
            case  4: qtKeyCode = Qt::Key_3; unicode  = 0x33; break;
            case  5: qtKeyCode = Qt::Key_4; unicode  = 0x34; break;
            case  6: qtKeyCode = Qt::Key_5; unicode  = 0x35; break;
            case  7: qtKeyCode = Qt::Key_6; unicode  = 0x36; break;
            case  8: qtKeyCode = Qt::Key_7; unicode  = 0x37; break;
            case  9: qtKeyCode = Qt::Key_8; unicode  = 0x38; break;
            case 10: qtKeyCode = Qt::Key_9; unicode  = 0x39; break;

            case 55: qtKeyCode = Qt::Key_Asterisk;
                       unicode  = 0x2A;
                       break;
            case 57: qtKeyCode = Qt::Key_NumberSign;
                       unicode  = 0x23;
                       break;

            // Navigation+
            case   33: qtKeyCode = Qt::Key_Call; break;
            case   34: qtKeyCode = Qt::Key_Hangup; break;
            case   46: qtKeyCode = Qt::Key_Context1; break;
            case   14: qtKeyCode = Qt::Key_Back; break;
            case  103: qtKeyCode = Qt::Key_Up; break;
            case  108: qtKeyCode = Qt::Key_Down; break;
            case  105: qtKeyCode = Qt::Key_Left; break;
            case  106: qtKeyCode = Qt::Key_Right; break;
            case   28: qtKeyCode = Qt::Key_Select; break;

            // Keys on left hand side of device
            case   30: qtKeyCode = Qt::Key_VolumeUp; break;
            case   48: qtKeyCode = Qt::Key_VolumeDown; break;

            // Keys on right hand side of device
//            case 0x31: qtKeyCode = Qt::Key_F7; break;   // Key +
  //          case 0x30: qtKeyCode = Qt::Key_F8; break;   // Key -

            // Camera
            case   35: qtKeyCode = Qt::Key_F4; break;
	    // Home
            case   18: qtKeyCode = Qt::Key_Home; break;
	    // Home
            case   32: qtKeyCode = Qt::Key_Context2; break;
        }

//        qLog(Input) << "processKeyEvent(): key=" << qtKeyCode << ", unicode=" << unicode;

        processKeyEvent(unicode, qtKeyCode, modifiers, isPressed, false);

        if (isPressed)
            beginAutoRepeat(unicode, qtKeyCode, modifiers);
        else
            endAutoRepeat();

#if 0
        if (isPressed)
        {
            m_repeatKeyCode = qtKeyCode;
            m_unicode = unicode;
            m_timer->start(400);
        }
        else
            m_timer->stop();
#endif
    }
}

#if 0
void mpx220KbdHandler::repeat()
{
//    processKeyEvent(m_unicode, m_repeatKeyCode, Qt::NoModifier, false, true);
    processKeyEvent(m_unicode, m_repeatKeyCode, Qt::NoModifier, true, true);
    m_timer->start(80);
}
#endif

#endif // QT_QWS_GREENPHONE
